!! ASP (c), 2004-2013, Matt Alvarado (malvarad@aer.com)
!! Based on MELAM of H.D.Steele (c) 2000-2004
!!
!! File Description:
!! unifacparamMattAq.h
!! Contains the data needed for UNIFAC calculation of 
!! organic activity coefficients in the aqueous phase.
!! Original file provided by Rob Griffin, UNH

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! UPDATE HISTORY							!!
!!									!!
!! Month  Year   Name              Description				!!
!! 07     2006   Matt Alvarado     Began Update History			!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

C         Include file : unifacparamMattAq.h
C
C         Purpose: Unifac parameters
C
C         Included in Unidriva.f
C
CNotes: 8 primary compounds: 
C             nC29
C             C4 diacid
C             naphthalene - 2,6-diacid
C             benzo(ghi) perylene
C             biomarker compound
C             4-carboxybenzoic acid
C             C18 acid
C             1,5-ditertbutyl-4,8-di(dimethylpropyl)-decalin
C       + 10 secondary compounds at 25C (See Pun et al., 2002) 
C       + 5 biomass burning compounds 
C         (Fig. 6 of Decesari et al, ACP 6, 375-402, 2006.)
C       (Data of Lyman, Reehl, Rosenblatt, 1990)
C
C       NMOL and NFUNC need to match DIMMOL and DIMFUN in unidrivb.f
C``````````````````````````````````````````````````````````````````````````````````````````````````````````````````````````````````
C revision History:  1. Developed by Betty Pun, AER, December, 1999 
C                 under CARB funding
C                   2. Modified by Betty Pun, AER, December, 1999 
C                      under CARB funding for Type B module
C                   3. updated for new POA mix and proper structures
C                      and for use in fortran, Rob Griffin, CIT 1/00
C		    4. order of compounds revised, Matt Alvarado, MIT, 4/06
C                   5. Added 5 biomass burning markers and water, 
C                      fixed errors, Matt Alvarado, MIT, 6/06
          REAL RG(20), QG(20), A(20,20)
          INTEGER NU(24,20)
C
C
C    no. of possible molecules      
          NMOL = 24
C
C
C   no. of functional groups
          NFUNC = 20
C
C
      DIMFUN = 20
C          max dimension of unifac parameter arrays for functional groups 
C          = NFUNC in unifacparamb.h
C 
      DIMMOL = 24
C          max dimension of unifac parameter arrays for functional groups 
C          = NMOL in unifacparamb.h
C    Z = 10 is a fixed parameter in Unifac
C
          Z = 10.0
C
         TINY = 1.0E-08
C
C       order for functional groups: CH3, CH2, CH, C, C=C, C=CH
C       ACH, AC, ACCH3, ACCH2, OH, ACOH, CH3CO, CHO, COOH, ACNO2, 
C       CH-O, CH2COO, CH2CO, H2O
C
C     group volume parameters
C     dimension of RG is the same as NFUNC
          DATA RG  /0.9011, 0.6744, 0.4469, 0.2195, 0.6605, 0.8886,
     +              0.5313, 0.3652, 1.2663, 1.0396, 1.0000, 0.8952,
     +              1.6724, 0.9980, 1.3013, 1.4199, 0.6908, 1.6764,
     +              1.4457, 0.9200/
C
C    group surface area parameters
C    dimension of QG is the same as NFUNC
          DATA QG  /0.8480, 0.5400, 0.2280, 0.0, 0.4850, 0.6760,
     +              0.4000, 0.1200, 0.9680, 0.6600, 1.2000, 0.6800,
     +              1.4880, 0.9480, 1.2240, 1.1040, 0.468, 1.420,
     +              1.180, 1.400/
C
C   no. of groups in each molecule (# CH3 in each 1-24, then
C    # CH2 in each 1-24, then # CH in each 1-24, etc.)
C    Line 1: P1, P2, P3, P4, P5, P6, P7, P8
C    Line 2: S1, S2, S3, S4, S5
C    Line 3: S6, S7, S8, S9, S10
C    Line 4: LEVO, CBIO, CPD1, CPD2, CPD3, H2O
C
c     start with CH3
c
      DATA  NU  /2, 0, 0, 0, 8, 0, 1, 12, 
     +      0, 1, 2, 2, 2, 
     +      0, 0, 0, 2, 3,
     +      0, 0, 0, 1, 0, 0,  
c        CH2
     +      27, 2, 0, 0, 11, 0, 16, 6, 
     +      0, 0, 0, 0, 2, 
     +      0, 0, 0, 12, 3,
     +      1, 2, 1, 5, 10, 0,
c        CH  
     +      0, 0, 0, 0, 6, 0, 0, 6,
     +      0, 0, 0, 2, 3, 
     +      0, 0, 0, 2, 3,
     +      3, 6, 1, 4, 1, 0,
c        c
     +      0, 0, 0, 0, 5, 0, 0, 4, 
     +      0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 1,
     +      0, 0, 0, 2, 0, 0,
c        C=C
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 1, 0, 0,
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 2, 0,
c         C=CH
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 2, 1, 1, 0,
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 1, 0, 0,	 
C       aroC (ACH)
     +      0, 0, 6, 12, 0, 4, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      1, 2, 6, 0, 0,
     +      0, 0, 6, 0, 2, 0,
C       pahC (AC)
     +      0, 0, 4, 10, 0, 2, 0, 0, 
     +      0, 0, 0, 0, 0, 
     +      1, 2, 2, 0, 0,
     +      0, 0, 0, 0, 3, 0,
c        tol (ACCH3)
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      2, 2, 1, 0, 0,
     +      0, 0, 0, 0, 0, 0,
c         EB (ACCH2)
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      0, 0, 1, 0, 0,
     +      0, 0, 2, 0, 0, 0,
c       OH
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 1, 1, 1,
     +      0, 0, 0, 1, 1,
     +      3, 6, 0, 1, 1, 0,
c      phen (ACOH)
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      1, 0, 0, 0, 0,
     +      0, 0, 4, 0, 1, 0,
c      ket (CH3CO)
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 1, 1,
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0, 0,
c     ald
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 1, 2, 0, 1,
     +      0, 1, 0, 0, 0,
     +      0, 0, 0, 0, 0, 0,
c      COOH
     +      0, 2, 2, 0, 0, 2, 1, 0, 
     +      2, 2, 0, 1, 0,
     +      1, 1, 0, 0, 0,
     +      0, 0, 0, 1, 4, 0, 
c      no2
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      1, 0, 1, 1, 1,
     +      0, 0, 0, 0, 0, 0,
c      CH-O
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0,
     +      2, 4, 0, 0, 1, 0,
c      CH2COO
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0,
     +      0, 0, 1, 0, 1, 0,
c      CH2CO
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 2, 3, 0,
c      H2O
     +      0, 0, 0, 0, 0, 0, 0, 0, 
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0,
     +      0, 0, 0, 0, 0, 1/ 

C
c   CH3 with 1-20 then CH2 with 1-20 then CH with 1-20
c   then C with 1-20, etc.
c
c    start with CH3
c
C  interaction parameters group-group
      DATA A / 0.0, 0.0, 0.0, 0.0, 2520.00, 2520.00, -11.1200, 
     +       -11.1200, -69.700, -69.700, 156.400, 10000.00,
     +       26.7600, 505.700, 315.300, 5541.00, 83.3600, 
     +       114.800, 26.7600, 300.00,
c       CH2
     +       0.0, 0.0, 0.0,
     +       0.0, 2520.00, 2520.00, -11.1200, -11.1200, -69.700,
     +       -69.700, 156.400, 10000.00, 26.7600, 505.700,
     +       315.300, 5541.00, 83.3600, 114.800, 26.7600, 300.00,
c       CH
     +       0.0, 0.0, 0.0, 0.0, 2520.00,
     +       2520.00, -11.1200, -11.1200, -69.700, -69.700,
     +       156.400, 10000.00, 26.7600, 505.700, 315.300,
     +       5541.00, 83.3600, 114.800, 26.7600, 300.00,
c        C
     +       0.0, 0.0, 0.0, 0.0, 2520.00, 2520.00,
     +       -11.1200, -11.1200, -69.700, -69.700, 156.400,
     +       10000.00, 26.7600, 505.700, 315.300, 5541.00,
     +       83.3600, 114.800, 26.7600, 300.00,
c        C=C
     +       -200.00, -200.00, -200.00, -200.00, 0.0, 0.0,
     +       -97.7800, -97.7800, -269.700, -269.700, 8694.00,
     +       732.200, -82.9200, 0.0, 349.200, 0.0, 76.4400, 
     +       362.400, -82.9200, 692.700,
c        C=CH
     +       -200.00,
     +       -200.00, -200.00, -200.00, 0.0, 0.0, -97.7800,
     +       -97.7800, -269.700, -269.700, 8694.00, 732.200,
     +       -82.9200, 0.0, 349.200, 0.0, 76.4400, 362.400, 
     +       -82.9200, 692.700,
c        aro (ACH)
     +       61.1300, 61.1300,
     +       61.1300, 61.1300, 340.700, 340.700, 0.0, 0.0,
     +       -146.800, -146.800, 89.600, 270.200, 140.100, 0.0,
     +       62.3200, 1824.00, 52.1300, 85.8400, 140.100, 362.300,
c        pah (AC)
     +       61.1300, 61.1300, 61.1300, 61.1300,
     +       340.700, 340.700, 0.0, 0.0, -146.800, -146.800, 89.600,
     +       270.200, 140.100, 0.0, 62.3200, 1824.00, 52.1300, 
     +       85.8400, 140.100, 362.300,
c        tol (ACCH3)
     +       76.500, 76.500, 76.500, 76.500, 4102.00, 4102.00, 167.00,
     +       167.00, 0.0, 0.0, 25.8200, 10000.00, 365.800, 0.0,
     +       268.200, -127.800, 65.6900, -170.00, 365.800, 377.600,
c        EB (ACCH2)
     +       76.500, 76.500, 76.500, 76.500,
     +       4102.00, 4102.00, 167.00, 167.00, 0.0, 0.0, 25.8200,
     +       10000.00, 365.800, 0.0, 268.200, -127.800, 65.6900, 
     +       -170.00, 365.800, 377.600,
c        OH
     +       986.500, 986.500, 986.500, 986.500, 693.900, 693.900,
     +       636.100, 636.100, 803.200, 803.200, 0.0, -274.500, 
     +       164.500, -404.800, -151.00, 561.600, 237.700, 
     +       245.400, 164.500, -229.100,
c       phenol (ACOH)
     +       912.200, 912.200,
     +       912.200, 912.200, 926.300, 926.300, 1174.00, 1174.00,
     +       674.300, 674.300, -442.100, 0.0, -246.800, 0.0, 0.0,
     +       0.0, 0.0, -72.5800, -246.800, 125.400,
c        ket (CH3CO)
     +       476.400, 476.400, 476.400, 476.400, 524.500,
     +       524.500, 25.7700, 25.7700, -52.100, -52.100, 84.00,
     +       -158.800, 0.0, 128.00, -297.800, 0.0, 52.3800, 
     +       372.200, 0.0, -195.400,
c        ald (CHO)
     +       677.00, 677.00,
     +       677.00, 677.00, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 441.800,
     +       0.0, -37.3600, 0.0, 0.0, 0.0, -7.83800, 0.0, -37.3600,
     +       -257.300,
c       COOH
     +       663.500, 663.500, 663.500,
     +       663.500, 730.400, 730.400, 537.400, 537.400, 603.800,
     +       603.800, 199.00, 0.0, 669.400, 0.0, 0.0, 0.0, 
     +       664.600, 660.200, 669.400, -14.0900,
c       AC-NO2
     +       543.00,
     +       543.00, 543.00, 543.00, 0.0, 0.0, 194.900, 194.900, 
     +       4448.00, 4448.00, 157.100, 0.0, 0.0, 0.0, 0.0, 
     +       0.0, 0.0, 0.0, 0.0, 399.500,
c       CH-O
     +       251.500, 251.500, 251.500, 251.500, 289.300, 289.300,
     +       32.1400, 32.1400,
     +       213.100, 213.100, 28.0600, 0.0, 5.20200, 304.100, 
     +       -338.500, 0.0,
     +       0.0, -235.700, 5.20200, 540.500,
c       CH2COO
     +       232.100, 232.100, 232.100, 232.100, 4.82600, 4.82600, 
     +       5.99400, 5.99400, 5688.00, 5688.00, 101.100, -442.800, 
     +       -213.700, 0.0, -256.300, 0.0, 
     +       461.300, 0.0, -213.700, 14.4200,
c       CH2CO
     +       476.400, 476.400, 476.400, 476.400, 524.500, 524.500, 
     +       25.7700, 25.7700, -52.100, -52.100, 84.00, -158.800, 
     +       0.0, 128.00, -297.800, 0.0, 
     +       52.3800, 372.200, 0.0, -195.400, 
C        H2O
     +       1318.00, 1318.00, 1318.00, 1318.00, 634.200, 634.200,
     +       903.800, 903.800, 5965.00, 5965.00, 353.500, 531.300, 
     +       472.5, 232.700, -66.1700, 360.700, -314.700, 10000.00,
     +       472.500, 0.0/
C
